/********************************************************************
//        NAME- TestComponentController
// DESCRIPTION- Constructor for the TestComponentController object
//      PASSED- messageElement (HTMLElement / string): this is the
//                  object that will contain the status messages
//              progressBar (ProgressBar): this is the progress bar
//                  that will be updated with the percent completion
//                  of the system ID
//              completeFunction (Function): the function that will
//                  be executed when the systemID is complete
//     RETURNS- 
//       NOTES- controls the running and status checking of a
//              system ID. Hooks into a message element and a
//              progress bar in order to display ID status
********************************************************************/
function TestComponentController()
{
  this.startCommand  = 'commands/startCatalog.php';
  this.statusCommand = 'commands/getProcessStatus.php';
  this.resultCommand = 'commands/getProcessResult.php';
  this.catalogResultCommand = 'commands/getCatalogResult.php';
  
  this.busyMessage    = translate('MSG_AI_IS_BUSY');

  this.tab = Frontend.tabs.helpTabs.tabs.testComponent;
  this.messageDiv = extendElement(getElement('testComponentMessages'));
  this.element = getElement('testComponentTableScroller');
  this.statusDiv = extendElement(getElement('testComponentStatus'));
  this.button = extendElement(getElement('continueCatalogButton'));

  this.loadingImg = 'images/status_unknown.gif';
  this.passedImg = 'images/status_passed.gif';
  this.failedImg = 'images/status_failed.gif';
  
  this.isCrashed = false;
  this.hasCrashedChild  = false;

  this.shadowBox = new ShadowBox(this, this.element);

  this.init();
}

TestComponentController.prototype.init = function init()
{
  this.messageDiv.hide();
  this.statusDiv.hide();
  this.button.style.position = 'absolute';
  this.button.centerHorizontally();
  this.button.controller = this;
  this.button.action = function ()
  {
    this.controller.getCatalogResult();
  }
}

/********************************************************************
//        NAME- TestComponentController.prototype.run
// DESCRIPTION- sends the ServerRequest to begin the system ID and
//              starts the status checking function
//      PASSED- 
//     RETURNS- 
//       NOTES- 
********************************************************************/
TestComponentController.prototype.run = function run()
{
  var request = new ServerRequest(this.startCommand);
  request.controller = this;
  request.callback = function(data)
  {
    this.controller.runResponse(data);
  }
  request.send();
  return this;
}

/********************************************************************
//        NAME- TestComponentController.prototype.runResponse
// DESCRIPTION- this is called by the initial ServerRequest created
//              by run()
//      PASSED- data (Object): this is the response from the server
//     RETURNS- 
//       NOTES- checks data for an error, if none found it extracts
//              the PID and runs beginUpdating()
********************************************************************/
TestComponentController.prototype.runResponse = function runResponse(data)
{
  if (data.name && data.name.indexOf('Error') >= 0)
  {
    Frontend.reportError(data.value, true);
  }
  else
  {
    switch (data.state)
    {
    case 'busy':
      Frontend.reportError(this.bustMessage, true);
      break;
    default:
      this.processID = data.processID;
      this.isCrashed = false;
      this.hasCrashedChild = false;
      this.beginUpdating();
    }
  }
  return this;
}

/********************************************************************
//        NAME- TestComponentController.prototype.beginUpdating
// DESCRIPTION- begins checking the status of the system ID
//      PASSED- 
//     RETURNS- 
//       NOTES- initializes the progress bar and message element,
//              then sends the first update ServerRequest, request
//              will call update() when finished
********************************************************************/
TestComponentController.prototype.beginUpdating = function beginUpdating()
{
  clearChildren(this.element);
  this.messageDiv.show();
  Body.adjustToViewport(this.element);
  this.statusRequest = new ServerRequest(this.statusCommand);
  this.statusRequest.addVariable('processID', this.processID);
  this.statusRequest.controller = this;
  this.statusRequest.callback = function(data)
  {
    this.controller.update(data);
  }
  this.statusRequest.send();
  return this;
}

/********************************************************************
//        NAME- TestComponentController.prototype.update
// DESCRIPTION- updates the hooked message element and progress bar
//              with the latest status of the System ID
//      PASSED- data (Object): the response from the server
//     RETURNS- 
//       NOTES- checks data for an error, finding none it extracts
//              the state and percent complete. If state is a final
//              state, the result is saved to idResult.xml, otherwise
//              updating continues
********************************************************************/
TestComponentController.prototype.update = function update(response)
{
  if (response.name && response.name.indexOf('Error') >= 0)
  {
    Frontend.reportError(response.value, true);
  }
  else
  {
    var key = response.key;
    var data = response.data;
    var state = data[key['state']];
    clearChildren(this.element);
    this.drawLoadingTable(key, data[key['children']]);
    var runningStates = ['canceling', 'running'];
    if (runningStates.contains(state))
    {
      var request = this.statusRequest;
      setTimeout(function()
      {
        request.send();
      }, 500);
    }
    else if (state == 'crashed')
    {
      this.isCrashed = true;
      Frontend.reportError(translate('MSG_ABNORMAL_TERMINIATION'), true);
      this.getResult();
    }
    else
    {
      if (this.hasCrashedChild == true)
      {
        this.statusDiv.show();
        Body.adjustToViewport(this.element);
      }
      else
      {
        this.getCatalogResult();
      }
    }
  }
  return this;
}

TestComponentController.prototype.drawLoadingTable = function drawLoadingTable(key, components)
{
  var numberOfComponents = components.length;
  var table = new HTMLTag('table', {'id':'testComponentLoadingTable'});
  var thead = table.appendChild(new HTMLTag('thead'));
  var tr = thead.appendChild(new HTMLTag('tr'));
  tr.appendChild(new HTMLTag('td', {'colspan':'3','text':(translate('MSG_TEST_COMPONENTS') + ':')}));
  var tbody = table.appendChild(new HTMLTag('tbody'));
  for (var i=0; i<numberOfComponents; i++)
  {
    var component = components[i];
    var name = component[key['component']];
    var state = component[key['state']];
    var tr = tbody.appendChild(new HTMLTag('tr'));
    tr.appendChild(new HTMLTag('th', {'text':name}));
    if (state == 'completed')
    {
      var td = tr.appendChild(new HTMLTag('td', {'class':'image'}));
      var img = td.appendChild(new HTMLTag('img', {'src':this.passedImg}));
      img.empty = true;
      tr.appendChild(new HTMLTag('td', {'text':translate('MSG_COMPLETED')}));
    }
    else if (state == 'crashed')
    {
      this.hasCrashedChild = true;
      var td = tr.appendChild(new HTMLTag('td', {'class':'image'}));
      var img = td.appendChild(new HTMLTag('img', {'src':this.failedImg}));
      img.empty = true;
      tr.appendChild(new HTMLTag('td', {'text':translate('MSG_ABNORMAL_TERMINATION')}));
    }
    else
    {
      var td = tr.appendChild(new HTMLTag('td', {'class':'image'}));
      var img = td.appendChild(new HTMLTag('img', {'src':this.loadingImg}));
      img.empty = true;
      tr.appendChild(new HTMLTag('td', {'text':(translate('MSG_LOADING') + '...')}));
    }
  }
  table.draw(this.element);
}

TestComponentController.prototype.drawCatalogResult = function drawCatalogResult(result)
{
  this.messageDiv.hide();
  this.statusDiv.hide();
  clearChildren(this.element);
  Body.adjustToViewport(this.element);
  this.shadowBox.show();
  var key = result.key;
  var data = result.data;
  var that = this;
  backgroundTask(function ()
  {
    var numberOfCatResults = data.length;
    var table = new HTMLTag('table', {'id':'testComponentTable', 'class':'linedTable'});
    for (var i=0; i<numberOfCatResults; i++)
    {
      var catResult = data[i];
      var tr = table.appendChild(new HTMLTag('tr', {'class':'header'}));
      tr.appendChild(new HTMLTag('th', {'colspan':'2','text':catResult[key['component']]}));
      var devices = catResult[key['devices']];
      var numberOfDevices = devices.length;
      for (var j=0; j<numberOfDevices; j++)
        that.drawDevice(devices[j], key, table);
    }
    table.draw(that.element);
    that.shadowBox.hide();
  });
}

TestComponentController.prototype.drawDevice = function drawDevice(device, key, table)
{
  var tr = table.appendChild(new HTMLTag('tr'));
  var th = tr.appendChild(new HTMLTag('th', {'colspan':'2','text':device[key['caption']]}));
  var tests = device[key['tests']];
  var numberOfTests = tests.length;
  for (var i=0; i<numberOfTests; i++)
    this.drawTest(tests[i], key, table);
}

TestComponentController.prototype.drawTest = function drawTest(test, key, table)
{
  var padding = '15px';
  var tr = table.appendChild(new HTMLTag('tr'));
  tr.appendChild(new HTMLTag('th', {'style':('padding-left: ' + padding),'text':test[key['caption']]}));
  tr.appendChild(new HTMLTag('td', {'text':test[key['description']]}));
  var parameters = test[key['parameters']];
  var numberOfParameters = parameters.length;
  for (var i=0; i<numberOfParameters; i++)
    this.drawParameter(parameters[i], key, table);
}

TestComponentController.prototype.drawParameter = function drawParameter(parameter, key, table)
{
  var padding = '30px';
  var tr = table.appendChild(new HTMLTag('tr'));
  tr.appendChild(new HTMLTag('td', {'style':('padding-left: ' + padding),'text':parameter[key['caption']]}));
  tr.appendChild(new HTMLTag('td', {'text':parameter[key['description']]}));
}


/********************************************************************
//        NAME- TestComponentController.prototype.getResult
// DESCRIPTION- sends a ServerRequest to get the result of the ID,
//              in a normal completion, this is done in saveFile(),
//              but we can perform getResult() in case of an abnormal
//              terminiation here to clear the shared memory
//      PASSED-
//     RETURNS- 
//       NOTES-
********************************************************************/
TestComponentController.prototype.getResult = function getResult()
{
  var request = new ServerRequest(this.resultCommand);
  request.addVariable('processID',this.processID);
  request.callback = function(){};
  request.send();
  return this;
}

TestComponentController.prototype.getCatalogResult = function getCatalogResult()
{
  var request = new ServerRequest(this.catalogResultCommand);
  request.addVariable('processID',this.processID);
  request.controller = this;
  request.callback = function(data)
  {
    this.controller.drawCatalogResult(data);
  }
  request.send();
  return this;
}

/********************************************************************
//        NAME- TestComponentController.prototype.saveFile
// DESCRIPTION- sends a ServerRequest to save the ID results to
//              idResult.xml
//      PASSED- 
//     RETURNS- 
//       NOTES- 
********************************************************************/
TestComponentController.prototype.saveFile = function saveFile()
{
  clearChildren(this.messageElement);
  this.messageElement.appendChild(newText(this.savingMessage));
  var request = new ServerRequest(this.saveCommand);
  request.addVariable('processID', this.processID);
  request.controller = this;
  request.callback = function(data)
  {
    this.controller.saveComplete(data);
  }
  //Frontend.sendRequest(request);
  request.send();
  return this;
}

/********************************************************************
//        NAME- TestComponentController.prototype.saveComplete
// DESCRIPTION- called after the file is saved
//      PASSED- data (Object): the response from the server
//     RETURNS- 
//       NOTES- checks for an error, finding none it calls complete()
********************************************************************/
TestComponentController.prototype.saveComplete = function saveComplete(data)
{
  if (data.name && data.name.indexOf('Error') >= 0)
  {
    clearChildren(this.messageElement);
    this.messageElement.appendChild(newText('(NT) '+data.name+': '+data.value));
  }
  else if (data == 0)
  {
    this.complete();
  }
  return this;
}

/********************************************************************
//        NAME- TestComponentController.prototype.complete
// DESCRIPTION- executes the completeFunction if there is one
//      PASSED- 
//     RETURNS- 
//       NOTES- 
********************************************************************/
TestComponentController.prototype.complete = function complete()
{
  if (this.completeFunction != undefined)
    this.completeFunction();
  return this;
}
